"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.matches = matches;
exports.getFocusables = _getFocusables;
exports.tryFocus = tryFocus;
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf3 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _flow = _interopRequireDefault(require("lodash/fp/flow"));

var _filter = _interopRequireDefault(require("lodash/fp/filter"));

var _identity = _interopRequireDefault(require("lodash/fp/identity"));

var _sortBy = _interopRequireDefault(require("lodash/fp/sortBy"));

var _indexOf = _interopRequireDefault(require("lodash/fp/indexOf"));

var _uiA11y = require("@instructure/ui-a11y");

/**
* Utility function to abstract away vendor-specific equivalents to HTMLel.matches
*/
function matches(el, selector, textContent) {
  var match = (el.matches || el.matchesSelector || el.mozMatchesSelector || el.msMatchesSelector || el.oMatchesSelector || el.webkitMatchesSelector).call(el, selector);
  return textContent ? match && el.innerText.includes(textContent) : match;
}
/**
* Returns a list of focusable HTML elements inside of parentEl that match selector
*/


function _getFocusables(parentEl, selector, textContent) {
  return (0, _flow.default)(_uiA11y.findTabbable, !selector ? _identity.default : (0, _filter.default)(function (el) {
    return matches(el, selector, textContent);
  }), (0, _sortBy.default)(function (el) {
    return el.tabIndex || 0;
  }))(_reactDom.default.findDOMNode(parentEl));
}
/**
* Safeguard against somehow focusing on already removed HTML elements
*/


function tryFocus(el) {
  try {
    el.focus();
  } catch (err) {
    // eslint-disable-next-line
    console.warn('You tried to focus ' + el + ' but it is not in the DOM anymore');
  }
}
/**
* Helper Component that is used to handle focus within its children
* It is used through a ref and directly calling the internal functions
*/


var FocusGroup =
/*#__PURE__*/
function (_Component) {
  (0, _inherits2.default)(FocusGroup, _Component);

  function FocusGroup() {
    var _getPrototypeOf2;

    var _this;

    (0, _classCallCheck2.default)(this, FocusGroup);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = (0, _possibleConstructorReturn2.default)(this, (_getPrototypeOf2 = (0, _getPrototypeOf3.default)(FocusGroup)).call.apply(_getPrototypeOf2, [this].concat(args)));
    _this.focusableCache = {};
    return _this;
  }

  (0, _createClass2.default)(FocusGroup, [{
    key: "getFocusables",

    /**
    * returns a list of focusable HTML elements
    * that are inside of FocusGroup & match selector
    */
    value: function getFocusables(selector, textContent) {
      if (!this.focusableCache[selector]) {
        this.focusableCache[selector] = _getFocusables(this, selector, textContent);
      }

      return this.focusableCache[selector];
    }
    /**
    * focuses the first matching HTML element
    */

  }, {
    key: "focusFirst",
    value: function focusFirst(selector, textContent) {
      var focusables = this.getFocusables(selector, textContent);

      if (focusables.length > 0) {
        tryFocus(focusables[0]);
      }
    }
    /**
    * focuses the last matching HTML element
    */

  }, {
    key: "focusLast",
    value: function focusLast(selector, textContent) {
      var lastFocusable = this.getFocusables(selector, textContent).slice(-1);

      if (lastFocusable.length === 1) {
        tryFocus(lastFocusable[0]);
      }
    }
    /**
    * returns if there's a matching HTML element after the currently focused one
    */

  }, {
    key: "nextExists",
    value: function nextExists(selector, textContent) {
      var focusables = this.getFocusables(selector, textContent);
      var index = (0, _indexOf.default)(document.activeElement, focusables);
      return focusables.length > 0 && index !== focusables.length - 1;
    }
    /**
    * focuses the matching HTML element after the currently focused one
    */

  }, {
    key: "focusNext",
    value: function focusNext(selector, textContent) {
      var focusables = this.getFocusables(selector, textContent);

      if (focusables.length === 0) {
        return;
      }

      var index = (0, _indexOf.default)(document.activeElement, focusables);

      if (index === -1) {
        tryFocus(focusables[0]);
      } else if (index === focusables.length - 1) {
        if (this.props.wrap) {
          tryFocus(focusables[0]);
        }
      } else {
        tryFocus(focusables[index + 1]);
      }
    }
    /**
    * returns if there's a matching HTML element before the currently focused one
    */

  }, {
    key: "previousExists",
    value: function previousExists(selector, textContent) {
      var focusables = this.getFocusables(selector, textContent);
      var index = (0, _indexOf.default)(document.activeElement, focusables);
      return focusables.length > 0 && index !== 0;
    }
    /**
    * focuses the matching HTML element before the currently focused one
    */

  }, {
    key: "focusPrevious",
    value: function focusPrevious(selector, textContent) {
      var focusables = this.getFocusables(selector, textContent);

      if (focusables.length === 0) {
        return;
      }

      var index = (0, _indexOf.default)(document.activeElement, focusables);

      if (index === -1) {
        tryFocus(focusables[focusables.length - 1]);
      } else if (index === 0) {
        if (this.props.wrap) {
          tryFocus(focusables[focusables.length - 1]);
        }
      } else {
        tryFocus(focusables[index - 1]);
      }
    }
    /**
    * render as the given Component
    */

  }, {
    key: "render",
    value: function render() {
      var Component = this.props.asComponent; // Clean cache

      this.focusableCache = {};
      return _react.default.createElement(Component, this.props.asProps, this.props.children);
    }
  }]);
  FocusGroup.displayName = "FocusGroup";
  return FocusGroup;
}(_react.Component);

exports.default = FocusGroup;
FocusGroup.propTypes = {
  /**
  * determines if all previous & next functions should wrap around
  */
  wrap: _propTypes.default.bool,

  /**
  * the Component to be rendered in place of FocusGroup
  */
  asComponent: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.func]),

  /**
  * the props being passed to asComponent
  */
  asProps: _propTypes.default.object,

  /**
  * the children of asComponent
  */
  children: _propTypes.default.node
};
FocusGroup.defaultProps = {
  wrap: false,
  asComponent: 'div',
  asProps: {},
  children: void 0
  /**
  * a cache of focusables as an object with selectors as keys
  * note: this is cleaned on each render
  */

};