"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _get2 = _interopRequireDefault(require("@babel/runtime/helpers/get"));

var _react = _interopRequireDefault(require("react"));

var _times3 = _interopRequireDefault(require("lodash/times"));

var _uuid = require("uuid");

var _instructureValidations = require("instructure-validations");

var _uiElements = require("@instructure/ui-elements");

var _interaction_slugs = require("../../interaction_slugs");

var _InteractionType2 = _interopRequireDefault(require("../InteractionType"));

var _validationHelpers = require("../../util/validationHelpers");

var _formatMessage = _interopRequireDefault(require("@instructure/quiz-i18n/lib/format-message"));

var CategorizationInteractionType =
/*#__PURE__*/
function (_InteractionType) {
  (0, _inherits2.default)(CategorizationInteractionType, _InteractionType);

  function CategorizationInteractionType(obj) {
    var _this;

    (0, _classCallCheck2.default)(this, CategorizationInteractionType);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(CategorizationInteractionType).call(this));
    _this.slug = _interaction_slugs.CATEGORIZATION_SLUG;
    _this.translatedName = (0, _formatMessage.default)('Categorization');
    _this.defaultProperties = {};

    _this.getDefaultScoringData = function (intData) {
      var i = 0;
      var categories = Object.keys(intData.categories).map(function (uuid) {
        var matchingDistractorId = Object.keys(intData.distractors)[i];
        i++;
        return {
          id: uuid,
          scoringAlgorithm: 'AllOrNothing',
          scoringData: {
            value: [matchingDistractorId]
          }
        };
      });
      return {
        score_method: 'all_or_nothing',
        // snake case for backend
        value: categories
      };
    };

    _this.getDefaultInteractionData = function () {
      var _categories, _distractors;

      var _times = (0, _times3.default)(4, _uuid.v4),
          _times2 = (0, _slicedToArray2.default)(_times, 4),
          categoryIdA = _times2[0],
          categoryIdB = _times2[1],
          distractorIdA = _times2[2],
          distractorIdB = _times2[3];

      return {
        categoryOrder: [categoryIdA, categoryIdB],
        categories: (_categories = {}, (0, _defineProperty2.default)(_categories, categoryIdA, {
          id: categoryIdA,
          itemBody: ''
        }), (0, _defineProperty2.default)(_categories, categoryIdB, {
          id: categoryIdB,
          itemBody: ''
        }), _categories),
        distractors: (_distractors = {}, (0, _defineProperty2.default)(_distractors, distractorIdA, {
          id: distractorIdA,
          itemBody: ''
        }), (0, _defineProperty2.default)(_distractors, distractorIdB, {
          id: distractorIdB,
          itemBody: ''
        }), _distractors)
      };
    };

    _this.getRenderedResponse = function (responseValue, interactionData) {
      var distractors = interactionData.distractors;
      return _react.default.createElement(_uiElements.Text, {
        color: "primary"
      }, responseValue.map(function (category) {
        var c = interactionData.categories[category.id];
        var categoryValue = c ? c.itemBody : '';
        var categoryDistractors = category.value.map(function (distractorId) {
          var distractorValue = distractors[distractorId];
          return distractorValue ? distractorValue.itemBody : '';
        }).join(', ');
        return (0, _formatMessage.default)('Category: {categoryValue} | Distractors: {categoryDistractors}', {
          categoryValue: categoryValue,
          categoryDistractors: categoryDistractors
        });
      }).join(', '));
    };

    (0, _get2.default)((0, _getPrototypeOf2.default)(CategorizationInteractionType.prototype), "initializeProps", (0, _assertThisInitialized2.default)(_this)).call((0, _assertThisInitialized2.default)(_this), obj);
    return _this;
  }

  (0, _createClass2.default)(CategorizationInteractionType, [{
    key: "getDefaultUserResponse",
    value: function getDefaultUserResponse(intData) {
      var categories = Object.keys(intData.categories).map(function (uuid) {
        return {
          id: uuid,
          value: [],
          type: 'ArrayText'
        };
      });
      return {
        value: categories
      };
    }
  }, {
    key: "hasResponse",
    value: function hasResponse(resp) {
      return resp.some(function (category) {
        return category.value.length > 0;
      });
    }
  }]);
  return CategorizationInteractionType;
}(_InteractionType2.default);

exports.default = CategorizationInteractionType;
CategorizationInteractionType.useNewValidations = true;

CategorizationInteractionType.validations = function (data) {
  return {
    interactionData: {
      categories: [(0, _instructureValidations.each)({
        itemBody: [(0, _instructureValidations.rule)('presence', {
          message: (0, _validationHelpers.presenceMessage)((0, _formatMessage.default)('Category'))
        })]
      }), (0, _instructureValidations.rule)('noDuplicates', {
        field: 'itemBody',
        message: (0, _validationHelpers.noDuplicatesMessage)((0, _formatMessage.default)('Category'))
      })],
      distractors: [(0, _instructureValidations.each)({
        itemBody: [(0, _instructureValidations.rule)('presence', {
          message: (0, _validationHelpers.presenceMessage)((0, _formatMessage.default)('Answer'))
        })]
      }), (0, _instructureValidations.rule)('noDuplicates', {
        field: 'itemBody',
        message: (0, _validationHelpers.noDuplicatesMessage)((0, _formatMessage.default)('Answer'))
      })]
    }
  };
};